@@ -1,7 +1,7 @@
 /* movemail foo bar -- move file foo to file bar,
    locking file foo the way /bin/mail respects.
    Copyright (C) 1986, 1992, 1993, 1994, 1996, 1999, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+                 2005, 2006, 2007, 2008, 2009, 2010  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -360,7 +360,7 @@ main (argc, argv)
       time_t touched_lock, now;
 #endif
 
-      if (setuid (getuid ()) < 0 || setregid (-1, real_gid) < 0)
+      if (setuid (getuid ()) < 0 || setegid (real_gid) < 0)
 	fatal ("Failed to drop privileges", 0, 0);
 
 #ifndef MAIL_USE_MMDF
@@ -387,7 +387,7 @@ main (argc, argv)
       if (outdesc < 0)
 	pfatal_with_name (outname);
 
-      if (setregid (-1, priv_gid) < 0)
+      if (setegid (priv_gid) < 0)
 	fatal ("Failed to regain privileges", 0, 0);
 
       /* This label exists so we can retry locking
@@ -484,7 +484,7 @@ main (argc, argv)
 #endif
 
       /* Prevent symlink attacks truncating other users' mailboxes */
-      if (setregid (-1, real_gid) < 0)
+      if (setegid (real_gid) < 0)
 	fatal ("Failed to drop privileges", 0, 0);
 
       /* Check to make sure no errors before we zap the inbox.  */
@@ -519,7 +519,7 @@ main (argc, argv)
 #endif /* not MAIL_USE_SYSTEM_LOCK */
 
       /* End of mailbox truncation */
-      if (setregid (-1, priv_gid) < 0)
+      if (setegid (priv_gid) < 0)
 	fatal ("Failed to regain privileges", 0, 0);
 
 #ifdef MAIL_USE_MAILLOCK
