@@ -1,16 +1,17 @@
-;;; dos-w32.el --- Functions shared among MS-DOS and W32 (NT/95) platforms
+;; dos-w32.el --- Functions shared among MS-DOS and W32 (NT/95) platforms
 
-;; Copyright (C) 1996 Free Software Foundation, Inc.
+;; Copyright (C) 1996, 2001, 2002, 2003, 2004, 2005,
+;;   2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 
 ;; Maintainer: Geoff Voelker <voelker@cs.washington.edu>
 ;; Keywords: internal
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,9 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -52,10 +51,9 @@
 					; Packers
     ("\\.\\(a\\|o\\|tar\\|z\\|gz\\|taz\\|jar\\)$" . t)
 					; Unix stuff
-    ("\\.tp[ulpw]$" . t)
-					; Borland Pascal stuff
-    ("[:/]tags$" . nil)
-					; Emacs TAGS file
+    ("\\.sx[dmicw]$" . t)		; OpenOffice.org
+    ("\\.tp[ulpw]$" . t)		; borland Pascal stuff
+    ("[:/]tags$" . nil)			; emacs TAGS file
     )
   "*Alist for distinguishing text files from binary files.
 Each element has the form (REGEXP . TYPE), where REGEXP is matched
@@ -79,7 +77,7 @@ against the file name, and TYPE is nil for text, t for binary.")
   (let ((match (find-buffer-file-type-match filename))
 	(code))
     (if (not match)
-	default-buffer-file-type
+	(default-value 'buffer-file-type)
       (setq code (cdr match))
       (cond ((memq code '(nil t)) code)
 	    ((and (symbolp code) (fboundp code))
@@ -88,10 +86,13 @@ against the file name, and TYPE is nil for text, t for binary.")
 (setq-default buffer-file-coding-system 'undecided-dos)
 
 (defun find-buffer-file-type-coding-system (command)
-  "Choose a coding system for a file operation.
-If COMMAND is `insert-file-contents', the coding system is chosen based
-upon the filename, the contents of `untranslated-filesystem-list' and
-`file-name-buffer-file-type-alist', and whether the file exists:
+  "Choose a coding system for a file operation in COMMAND.
+COMMAND is a list that specifies the operation, an I/O primitive, as its
+CAR, and the arguments that might be given to that operation as its CDR.
+If operation is `insert-file-contents', the coding system is chosen based
+upon the filename (the CAR of the arguments beyond the operation), the contents
+of `untranslated-filesystem-list' and `file-name-buffer-file-type-alist',
+and whether the file exists:
 
   If it matches in `untranslated-filesystem-list':
     If the file exists:					`undecided'
@@ -101,9 +102,13 @@ upon the filename, the contents of `untranslated-filesystem-list' and
     If the match is nil (for dos-text):			`undecided-dos'
   Otherwise:
     If the file exists:					`undecided'
-    If the file does not exist:	       default-buffer-file-coding-system
+    If the file does not exist   default value of `buffer-file-coding-system'
 
-If COMMAND is `write-region', the coding system is chosen based upon
+Note that the CAR of arguments to `insert-file-contents' operation could
+be a cons cell of the form \(FILENAME . BUFFER\), where BUFFER is a buffer
+into which the file's contents were already read, but not yet decoded.
+
+If operation is `write-region', the coding system is chosen based upon
 the value of `buffer-file-coding-system' and `buffer-file-type'. If
 `buffer-file-coding-system' is non-nil, its value is used.  If it is
 nil and `buffer-file-type' is t, the coding system is `no-conversion'.
@@ -121,11 +126,22 @@ set to the appropriate coding system, and the value of
 `buffer-file-coding-system' will be used when writing the file."
 
   (let ((op (nth 0 command))
-	(target)
 	(binary nil) (text nil)
-	(undecided nil) (undecided-unix nil))
-    (cond ((eq op 'insert-file-contents) 
+	(undecided nil) (undecided-unix nil)
+	target target-buf)
+    (cond ((eq op 'insert-file-contents)
 	   (setq target (nth 1 command))
+	   ;; If TARGET is a cons cell, it has the form (FILENAME . BUFFER),
+	   ;; where BUFFER is a buffer into which the file was already read,
+	   ;; but its contents were not yet decoded.  (This form of the
+	   ;; arguments is used, e.g., in arc-mode.el.)  This function
+	   ;; doesn't care about the contents, it only looks at the file's
+	   ;; name, which is the CAR of the cons cell.
+	   (when (consp target)
+	     (setq target-buf
+		   (and (bufferp (cdr target))
+			(buffer-name (cdr target))))
+	     (setq target (car target)))
 	   ;; First check for a file name that indicates
 	   ;; it is truly binary.
 	   (setq binary (find-buffer-file-type target))
@@ -134,7 +150,17 @@ set to the appropriate coding system, and the value of
 		 ((find-buffer-file-type-match target)
 		  (setq text t))
 		 ;; For any other existing file, decide based on contents.
-		 ((file-exists-p target)
+		 ((or
+		   (file-exists-p target)
+		   ;; If TARGET does not exist as a file, replace its
+		   ;; base name with TARGET-BUF and try again.  This
+		   ;; is for jka-compr's sake, which strips the
+		   ;; compression (.gz etc.) extension from the
+		   ;; FILENAME, but leaves it in the BUFFER's name.
+		   (and (stringp target-buf)
+			(file-exists-p
+			 (expand-file-name target-buf
+					   (file-name-directory target)))))
 		  (setq undecided t))
 		 ;; Next check for a non-DOS file system.
 		 ((untranslated-file-p target)
@@ -143,8 +169,8 @@ set to the appropriate coding system, and the value of
 		 (text '(undecided-dos . undecided-dos))
 		 (undecided-unix '(undecided-unix . undecided-unix))
 		 (undecided '(undecided . undecided))
-		 (t (cons default-buffer-file-coding-system
-			  default-buffer-file-coding-system))))
+		 (t (cons (default-value 'buffer-file-coding-system)
+			  (default-value 'buffer-file-coding-system)))))
 	  ((eq op 'write-region)
 	   (if buffer-file-coding-system
 	       (cons buffer-file-coding-system
@@ -158,21 +184,20 @@ set to the appropriate coding system, and the value of
 
 (modify-coding-system-alist 'file "" 'find-buffer-file-type-coding-system)
 
-(defun find-file-binary (filename) 
+(defun find-file-binary (filename)
   "Visit file FILENAME and treat it as binary."
   (interactive "FFind file binary: ")
   (let ((file-name-buffer-file-type-alist '(("" . t))))
     (find-file filename)))
 
-(defun find-file-text (filename) 
+(defun find-file-text (filename)
   "Visit file FILENAME and treat it as a text file."
   (interactive "FFind file text: ")
   (let ((file-name-buffer-file-type-alist '(("" . nil))))
     (find-file filename)))
 
 (defun find-file-not-found-set-buffer-file-coding-system ()
-  (save-excursion
-    (set-buffer (current-buffer))
+  (with-current-buffer (current-buffer)
     (let ((coding buffer-file-coding-system))
       ;; buffer-file-coding-system is already set by
       ;; find-operation-coding-system, which was called from
@@ -186,24 +211,24 @@ set to the appropriate coding system, and the value of
       (setq buffer-file-type (eq buffer-file-coding-system 'no-conversion)))))
 
 ;;; To set the default coding system on new files.
-(add-hook 'find-file-not-found-hooks 
+(add-hook 'find-file-not-found-functions
 	  'find-file-not-found-set-buffer-file-coding-system)
 
-;;; To accomodate filesystems that do not require CR/LF translation.
+;;; To accommodate filesystems that do not require CR/LF translation.
 (defvar untranslated-filesystem-list nil
-  "List of filesystems that require no CR/LF translation when reading 
+  "List of filesystems that require no CR/LF translation when reading
 and writing files.  Each filesystem in the list is a string naming
 the directory prefix corresponding to the filesystem.")
 
 (defun untranslated-canonical-name (filename)
   "Return FILENAME in a canonicalized form for use with the functions
 dealing with untranslated filesystems."
-  (if (memq system-type '(ms-dos windows-nt))
+  (if (memq system-type '(ms-dos windows-nt cygwin))
       ;; The canonical form for DOS/W32 is with A-Z downcased and all
       ;; directory separators changed to directory-sep-char.
       (let ((name nil))
-	(setq name (mapconcat 
-		    '(lambda (char) 
+	(setq name (mapconcat
+		    '(lambda (char)
 		       (if (and (<= ?A char) (<= char ?Z))
 			   (char-to-string (+ (- char ?A) ?a))
 			 (char-to-string char)))
@@ -219,7 +244,7 @@ dealing with untranslated filesystems."
     filename))
 
 (defun untranslated-file-p (filename)
-  "Return t if FILENAME is on a filesystem that does not require 
+  "Return t if FILENAME is on a filesystem that does not require
 CR/LF translation, and nil otherwise."
   (let ((fs (untranslated-canonical-name filename))
 	(ufs-list untranslated-filesystem-list)
@@ -233,7 +258,7 @@ CR/LF translation, and nil otherwise."
 (defun add-untranslated-filesystem (filesystem)
   "Add FILESYSTEM to the list of filesystems that do not require
 CR/LF translation.  FILESYSTEM is a string containing the directory
-prefix corresponding to the filesystem.  For example, for a Unix 
+prefix corresponding to the filesystem.  For example, for a Unix
 filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
   ;; We use "D", not "f", to avoid confusing the user: "f" prompts
   ;; with a directory, but RET returns the current buffer's file, not
@@ -246,12 +271,12 @@ filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
 	    (cons fs untranslated-filesystem-list)))))
 
 (defun remove-untranslated-filesystem (filesystem)
-  "Remove FILESYSTEM from the list of filesystems that do not require 
+  "Remove FILESYSTEM from the list of filesystems that do not require
 CR/LF translation.  FILESYSTEM is a string containing the directory
-prefix corresponding to the filesystem.  For example, for a Unix 
+prefix corresponding to the filesystem.  For example, for a Unix
 filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
   (interactive "fUntranslated file system: ")
-  (setq untranslated-filesystem-list 
+  (setq untranslated-filesystem-list
 	(delete (untranslated-canonical-name filesystem)
 		untranslated-filesystem-list)))
 
@@ -353,6 +378,8 @@ filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
 
 (defvar printer-name)
 
+(declare-function default-printer-name "w32fns.c")
+
 (defun direct-print-region-function (start end
 					   &optional lpr-prog
 					   delete-text buf display
@@ -379,13 +406,16 @@ indicates a specific program should be invoked."
 	 (printer (or (and (boundp 'dos-printer)
 			   (stringp (symbol-value 'dos-printer))
 			   (symbol-value 'dos-printer))
-		      printer-name)))
+		      printer-name
+		      (default-printer-name))))
     (or (eq coding-system-for-write 'no-conversion)
 	(setq coding-system-for-write
 	      (aref eol-type 1)))	; force conversion to DOS EOLs
     (direct-print-region-helper printer start end lpr-prog
 				delete-text buf display rest)))
 
+(defvar print-region-function)
+(defvar lpr-headers-switches)
 (setq print-region-function 'direct-print-region-function)
 
 ;; Set this to nil if you have a port of the `pr' program
@@ -412,10 +442,12 @@ indicates a specific program should be invoked."
   (let ((printer (or (and (boundp 'dos-ps-printer)
 			  (stringp (symbol-value 'dos-ps-printer))
 			  (symbol-value 'dos-ps-printer))
-		     ps-printer-name)))
+		     ps-printer-name
+		     (default-printer-name))))
     (direct-print-region-helper printer start end lpr-prog
 				delete-text buf display rest)))
 
+(defvar ps-print-region-function)
 (setq ps-print-region-function 'direct-ps-print-region-function)
 
 ;(setq ps-lpr-command "gs")
@@ -425,4 +457,5 @@ indicates a specific program should be invoked."
 
 (provide 'dos-w32)
 
+;; arch-tag: dcfefdd2-362f-4fbc-9141-9634f5f4d6a7
 ;;; dos-w32.el ends here
