@@ -1,19 +1,20 @@
-;;; texinfo.el --- major mode for editing Texinfo files
+;;; texinfo.el --- major mode for editing Texinfo files -*- coding: utf-8 -*-
 
-;; Copyright (C) 1985, '88, '89, '90, '91,
-;;                '92, '93, '96, '97 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1988, 1989, 1990, 1991, 1992, 1993, 1996, 1997,
+;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+;;   Free Software Foundation, Inc.
 
 ;; Author: Robert J. Chassell
-;; Date:   [Set date below for texinfo-version]
-;; Maintainer: bug-texinfo@prep.ai.mit.edu
+;; Date:   [See date below for texinfo-version]
+;; Maintainer: FSF
 ;; Keywords: maint, tex, docs
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,36 +22,42 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+
+;;; Todo:
+
+;; - facemenu support.
+;; - command completion.
+
+;;; Commentary:
 
 ;;; Code:
 
-(or (fboundp 'defgroup)
-    (defmacro defgroup (&rest ignore) nil))
-
-(or (fboundp 'defcustom)
-    (defmacro defcustom (var value doc &rest ignore)
-      `(defvar ,var ,value ,doc)))
+(eval-when-compile (require 'tex-mode) (require 'cl))
+(defvar outline-heading-alist)
 
 (defgroup texinfo nil
-  "Texinfo Mode"
+  "Texinfo Mode."
+  :link '(custom-group-link :tag "Font Lock Faces group" font-lock-faces)
   :group 'docs)
 
-(defvar texinfo-version "2.37 of 24 May 1997")
+;;;###autoload
+(defcustom texinfo-open-quote (purecopy "``")
+  "String inserted by typing \\[texinfo-insert-quote] to open a quotation."
+  :type 'string
+  :group 'texinfo)
 
-(defun texinfo-version (&optional here)
-  "Show the version of texinfo.el in the minibuffer.
-If optional argument HERE is non-nil, insert info at point."
-  (interactive "P")
-  (let ((version-string 
-         (format "Version of \`texinfo.el\': %s" texinfo-version)))
-    (if here 
-        (insert version-string)
-      (if (interactive-p)
-          (message "%s" version-string)
-        version-string))))
+;;;###autoload
+(defcustom texinfo-close-quote (purecopy "''")
+  "String inserted by typing \\[texinfo-insert-quote] to close a quotation."
+  :type 'string
+  :group 'texinfo)
+
+(defcustom texinfo-mode-hook nil
+  "Normal hook run when entering Texinfo mode."
+  :type 'hook
+  :options '(turn-on-auto-fill flyspell-mode)
+  :group 'texinfo)
 
 
 ;;; Autoloads:
@@ -93,15 +100,15 @@ marked region.
 The functions for creating or updating nodes and menus, and their
 keybindings, are:
 
-    texinfo-update-node (&optional region-p)    \\[texinfo-update-node]
-    texinfo-every-node-update ()                \\[texinfo-every-node-update]
-    texinfo-sequential-node-update (&optional region-p)
+    `texinfo-update-node' (&optional region-p)    \\[texinfo-update-node]
+    `texinfo-every-node-update' ()                \\[texinfo-every-node-update]
+    `texinfo-sequential-node-update' (&optional region-p)
 
-    texinfo-make-menu (&optional region-p)      \\[texinfo-make-menu]
-    texinfo-all-menus-update ()                 \\[texinfo-all-menus-update]
-    texinfo-master-menu ()
+    `texinfo-make-menu' (&optional region-p)      \\[texinfo-make-menu]
+    `texinfo-all-menus-update' ()                 \\[texinfo-all-menus-update]
+    `texinfo-master-menu' ()
 
-    texinfo-indent-menu-description (column &optional region-p)
+    `texinfo-indent-menu-description' (column &optional region-p)
 
 The `texinfo-column-for-description' variable specifies the column to
 which menu descriptions are indented. Its default value is 32."
@@ -257,86 +264,130 @@ chapter."
 
 ;;; Don't you dare insert any `require' calls at top level in this file--rms.
 
+(defvar texinfo-section-list
+  '(("top" 1)
+    ("chapter" 2)
+    ("section" 3)
+    ("subsection" 4)
+    ("subsubsection" 5)
+    ("unnumbered" 2)
+    ("unnumberedsec" 3)
+    ("unnumberedsubsec" 4)
+    ("unnumberedsubsubsec" 5)
+    ("appendix" 2)
+    ("appendixsec" 3)
+    ("appendixsection" 3)
+    ("appendixsubsec" 4)
+    ("appendixsubsubsec" 5)
+    ("majorheading" 2)
+    ("chapheading" 2)
+    ("heading" 3)
+    ("subheading" 4)
+    ("subsubheading" 5))
+  "Alist of sectioning commands and their relative level.")
+
 ;;; Syntax table
 
-(defvar texinfo-mode-syntax-table nil)
-
-(if texinfo-mode-syntax-table
-    nil
-  (setq texinfo-mode-syntax-table (make-syntax-table))
-  (modify-syntax-entry ?\" " " texinfo-mode-syntax-table)
-  (modify-syntax-entry ?\\ " " texinfo-mode-syntax-table)
-  (modify-syntax-entry ?@ "\\" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?\^q "\\" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?\[ "(]" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?\] ")[" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?{ "(}" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?} "){" texinfo-mode-syntax-table)
-  (modify-syntax-entry ?\' "w" texinfo-mode-syntax-table))
+(defvar texinfo-mode-syntax-table
+  (let ((st (make-syntax-table)))
+    (modify-syntax-entry ?\" "." st)
+    (modify-syntax-entry ?\\ "." st)
+    (modify-syntax-entry ?@ "\\" st)
+    (modify-syntax-entry ?\^q "\\" st)
+    (modify-syntax-entry ?\[ "(]" st)
+    (modify-syntax-entry ?\] ")[" st)
+    (modify-syntax-entry ?{ "(}" st)
+    (modify-syntax-entry ?} "){" st)
+    (modify-syntax-entry ?\n ">" st)
+    (modify-syntax-entry ?\' "w" st)
+    st))
 
 ;; Written by Wolfgang Bangerth <zcg51122@rpool1.rus.uni-stuttgart.de>
 ;; To override this example, set either `imenu-generic-expression'
 ;; or `imenu-create-index-function'.
 (defvar texinfo-imenu-generic-expression
-  '((nil "^@node[ \t]+\\([^,\n]*\\)" 1)
+  '((nil "^@\\(node\\|anchor\\)[ \t]+\\([^,\n]*\\)" 2)
     ("Chapters" "^@chapter[ \t]+\\(.*\\)$" 1))
+  "Imenu generic expression for Texinfo mode.  See `imenu-generic-expression'.")
 
-  "Imenu generic expression for TexInfo mode.  See `imenu-generic-expression'.")
+(defvar texinfo-font-lock-syntactic-keywords
+  '(("\\(@\\)c\\(omment\\)?\\>" (1 "<"))
+    ("^\\(@\\)ignore\\>" (1 "< b"))
+    ("^@end ignore\\(\n\\)" (1 "> b")))
+  "Syntactic keywords to catch comment delimiters in `texinfo-mode'.")
+
+(defconst texinfo-environments
+  '("cartouche" "copying" "defcv" "deffn" "defivar" "defmac"
+    "defmethod" "defop" "defopt" "defspec" "deftp" "deftypefn"
+    "deftypefun" "deftypevar" "deftypevr" "defun" "defvar"
+    "defvr" "description" "detailmenu" "direntry" "display"
+    "documentdescription" "enumerate" "example" "flushleft"
+    "flushright" "format" "ftable" "group" "ifclear" "ifset"
+    "ifhtml" "ifinfo" "ifnothtml" "ifnotinfo" "ifnotplaintext"
+    "ifnottex" "ifplaintext" "iftex" "ignore" "itemize" "lisp"
+    "macro" "menu" "multitable" "quotation" "smalldisplay"
+    "smallexample" "smallformat" "smalllisp" "table" "tex"
+    "titlepage" "verbatim" "vtable")
+  "List of Texinfo environments.")
+
+(defconst texinfo-environment-regexp
+  (concat "^@" (regexp-opt (cons "end" texinfo-environments) t) "\\>")
+  "Regexp for environment-like Texinfo list commands.
+Subexpression 1 is what goes into the corresponding `@end' statement.")
+
+(defface texinfo-heading
+  '((t (:inherit font-lock-function-name-face)))
+  "Face used for section headings in `texinfo-mode'."
+  :group 'texinfo)
+(define-obsolete-face-alias 'texinfo-heading-face 'texinfo-heading "22.1")
+(defvar texinfo-heading-face 'texinfo-heading)
 
 (defvar texinfo-font-lock-keywords
-  '(;; All but the first 2 had an OVERRIDE of t.
+  `(;; All but the first had an OVERRIDE of t.
     ;; It didn't seem to be any better, and it's slower--simon.
-    ("^\\(@c\\|@comment\\)\\>.*" . font-lock-comment-face) ;comments
-    ;; Robert J. Chassell <bob@gnu.ai.mit.edu> says remove this line.
+    ;; Robert J. Chassell <bob@gnu.org> says remove this line.
     ;;("\\$\\([^$]*\\)\\$" 1 font-lock-string-face t)
     ("@\\([a-zA-Z]+\\|[^ \t\n]\\)" 1 font-lock-keyword-face) ;commands
-    ("^\\*\\(.*\\)[\t ]*$" 1 font-lock-function-name-face t) ;menu items
-    ("@\\(emph\\|strong\\|b\\|i\\){\\([^}]+\\)" 2 font-lock-comment-face)
-    ("@\\(file\\|kbd\\|key\\|url\\|email\\){\\([^}]+\\)" 2 font-lock-string-face)
-    ("@\\(samp\\|code\\|var\\|math\\){\\([^}]+\\)"
-     2 font-lock-variable-name-face)
-    ("@\\(cite\\|xref\\|pxref\\){\\([^}]+\\)" 2 font-lock-constant-face)
-    ("@\\(end\\|itemx?\\) +\\(.+\\)" 2 font-lock-function-name-face keep)
-    )
-  "Additional expressions to highlight in TeXinfo mode.")
+    ("^\\*\\([^\n:]*\\)" 1 font-lock-function-name-face t) ;menu items
+    ("@\\(emph\\|i\\|sc\\){\\([^}]+\\)" 2 'italic)
+    ("@\\(strong\\|b\\){\\([^}]+\\)" 2 'bold)
+    ("@\\(kbd\\|key\\|url\\|uref\\){\\([^}]+\\)" 2 font-lock-string-face)
+    ;; The following two groups have an OVERRIDE of `keep' because
+    ;; their arguments frequently include a @@, and we don't want that
+    ;; to overwrite the normal fontification of the argument.
+    ("@\\(file\\|email\\){\\([^}]+\\)" 2 font-lock-string-face keep)
+    ("@\\(samp\\|code\\|var\\|math\\|env\\|command\\|option\\){\\([^}]+\\)"
+     2 font-lock-variable-name-face keep)
+    ("@\\(cite\\|x?ref\\|pxref\\|dfn\\|inforef\\){\\([^}]+\\)"
+     2 font-lock-constant-face)
+    ("@\\(anchor\\){\\([^}]+\\)" 2 font-lock-type-face)
+    ("@\\(dmn\\|acronym\\|value\\){\\([^}]+\\)" 2 font-lock-builtin-face)
+    ("@\\(end\\|itemx?\\) +\\(.+\\)" 2 font-lock-keyword-face keep)
+    ;; (,texinfo-environment-regexp
+    ;;  1 (texinfo-clone-environment (match-beginning 1) (match-end 1)) keep)
+    (,(concat "^@" (regexp-opt (mapcar 'car texinfo-section-list) t)
+	       ".*\n") 0 texinfo-heading-face t))
+  "Additional expressions to highlight in Texinfo mode.")
 
-(defvar texinfo-section-list
-  '(("top" 1)
-    ("majorheading" 1)
-    ("chapter" 2)
-    ("unnumbered" 2)
-    ("appendix" 2)
-    ("chapheading" 2)
-    ("section" 3)
-    ("unnumberedsec" 3)
-    ("appendixsec" 3)
-    ("heading" 3)
-    ("subsection" 4)
-    ("unnumberedsubsec" 4)
-    ("appendixsubsec" 4)
-    ("subheading" 4)
-    ("subsubsection" 5)
-    ("unnumberedsubsubsec" 5)
-    ("appendixsubsubsec" 5)
-    ("subsubheading" 5))
-  "Alist of sectioning commands and their relative level.")
-
-(defun texinfo-outline-level ()
-  ;; Calculate level of current texinfo outline heading.
-  (save-excursion
-    (if (bobp)
-        0
-      (forward-char 1)
-      (let* ((word (buffer-substring-no-properties
-                    (point) (progn (forward-word 1) (point))))
-             (entry (assoc word texinfo-section-list)))
-        (if entry
-            (nth 1 entry)
-          5)))))
+(defun texinfo-clone-environment (start end)
+  (let ((endp nil))
+    (save-excursion
+      (ignore-errors
+	(goto-char start)
+	(when (looking-at "end\\Sw+\\(\\sw+\\)")
+	  (setq endp t start (match-beginning 1) end (match-end 1)))
+	(unless (get-char-property start 'text-clones)
+	  (if endp
+	      (texinfo-last-unended-begin)
+	    (forward-word 1)
+	    (texinfo-next-unmatched-end))
+	  (skip-syntax-forward "^w")
+	  (when (looking-at
+		 (concat (regexp-quote (buffer-substring start end)) "\\>"))
+	    (text-clone-create start end 'spread "\\w*")))))))
 
 
 ;;; Keybindings
-(defvar texinfo-mode-map nil)
 
 ;;; Keys common both to Texinfo mode and to TeX shell.
 
@@ -357,66 +408,101 @@ chapter."
 ;; Mode documentation displays commands in reverse order
 ;; from how they are listed in the texinfo-mode-map.
 
-(if texinfo-mode-map
-    nil
-  (setq texinfo-mode-map (make-sparse-keymap))
+(defvar texinfo-mode-map
+  (let ((map (make-sparse-keymap)))
 
-  ;; bindings for `texnfo-tex.el'
-  (texinfo-define-common-keys texinfo-mode-map)
+    ;; bindings for `texnfo-tex.el'
+    (texinfo-define-common-keys map)
 
-  ;; bindings for `makeinfo.el'
-  (define-key texinfo-mode-map "\C-c\C-m\C-k" 'kill-compilation)
-  (define-key texinfo-mode-map "\C-c\C-m\C-l"
-    'makeinfo-recenter-compilation-buffer)
-  (define-key texinfo-mode-map "\C-c\C-m\C-r" 'makeinfo-region)
-  (define-key texinfo-mode-map "\C-c\C-m\C-b" 'makeinfo-buffer)
+    (define-key map "\"" 'texinfo-insert-quote)
 
-  ;; bindings for `texinfmt.el'
-  (define-key texinfo-mode-map "\C-c\C-e\C-r"    'texinfo-format-region)
-  (define-key texinfo-mode-map "\C-c\C-e\C-b"    'texinfo-format-buffer)
+    ;; bindings for `makeinfo.el'
+    (define-key map "\C-c\C-m\C-k" 'kill-compilation)
+    (define-key map "\C-c\C-m\C-l"
+      'makeinfo-recenter-compilation-buffer)
+    (define-key map "\C-c\C-m\C-r" 'makeinfo-region)
+    (define-key map "\C-c\C-m\C-b" 'makeinfo-buffer)
 
-  ;; bindings for updating nodes and menus
+    ;; bindings for `texinfmt.el'
+    (define-key map "\C-c\C-e\C-r"    'texinfo-format-region)
+    (define-key map "\C-c\C-e\C-b"    'texinfo-format-buffer)
 
-  (define-key texinfo-mode-map "\C-c\C-um"   'texinfo-master-menu)
+    ;; AUCTeX-like bindings
+    (define-key map "\e\r"		'texinfo-insert-@item)
 
-  (define-key texinfo-mode-map "\C-c\C-u\C-m"   'texinfo-make-menu)
-  (define-key texinfo-mode-map "\C-c\C-u\C-n"   'texinfo-update-node)
-  (define-key texinfo-mode-map "\C-c\C-u\C-e"   'texinfo-every-node-update)
-  (define-key texinfo-mode-map "\C-c\C-u\C-a"   'texinfo-all-menus-update)
+    ;; bindings for updating nodes and menus
 
-  (define-key texinfo-mode-map "\C-c\C-s"     'texinfo-show-structure)
+    (define-key map "\C-c\C-um"   'texinfo-master-menu)
 
-  (define-key texinfo-mode-map "\C-c}"          'up-list)
-  (define-key texinfo-mode-map "\C-c]"          'up-list)
-  (define-key texinfo-mode-map "\C-c{"          'texinfo-insert-braces)
+    (define-key map "\C-c\C-u\C-m"   'texinfo-make-menu)
+    (define-key map "\C-c\C-u\C-n"   'texinfo-update-node)
+    (define-key map "\C-c\C-u\C-e"   'texinfo-every-node-update)
+    (define-key map "\C-c\C-u\C-a"   'texinfo-all-menus-update)
 
-  ;; bindings for inserting strings
+    (define-key map "\C-c\C-s"     'texinfo-show-structure)
 
-  (define-key texinfo-mode-map "\C-c\C-c\C-d" 'texinfo-start-menu-description)
+    (define-key map "\C-c}"          'up-list)
+    (define-key map "\C-c]"          'up-list)
+    (define-key map "\C-c{"		'texinfo-insert-braces)
 
-  (define-key texinfo-mode-map "\C-c\C-cv"    'texinfo-insert-@var)
-  (define-key texinfo-mode-map "\C-c\C-ct"    'texinfo-insert-@table)
-  (define-key texinfo-mode-map "\C-c\C-cs"    'texinfo-insert-@samp)
-  (define-key texinfo-mode-map "\C-c\C-co"    'texinfo-insert-@noindent)
-  (define-key texinfo-mode-map "\C-c\C-cn"    'texinfo-insert-@node)
-  (define-key texinfo-mode-map "\C-c\C-ck"    'texinfo-insert-@kbd)
-  (define-key texinfo-mode-map "\C-c\C-ci"    'texinfo-insert-@item)
-  (define-key texinfo-mode-map "\C-c\C-cf"    'texinfo-insert-@file)
-  (define-key texinfo-mode-map "\C-c\C-cx"    'texinfo-insert-@example)
-  (define-key texinfo-mode-map "\C-c\C-ce"    'texinfo-insert-@end)
-  (define-key texinfo-mode-map "\C-c\C-cd"    'texinfo-insert-@dfn)
-  (define-key texinfo-mode-map "\C-c\C-cc"    'texinfo-insert-@code))
+    ;; bindings for inserting strings
+    (define-key map "\C-c\C-o"     'texinfo-insert-block)
+    (define-key map "\C-c\C-c\C-d" 'texinfo-start-menu-description)
+    (define-key map "\C-c\C-c\C-s" 'texinfo-insert-@strong)
+    (define-key map "\C-c\C-c\C-e" 'texinfo-insert-@emph)
+
+    (define-key map "\C-c\C-cv"    'texinfo-insert-@var)
+    (define-key map "\C-c\C-cu"    'texinfo-insert-@uref)
+    (define-key map "\C-c\C-ct"    'texinfo-insert-@table)
+    (define-key map "\C-c\C-cs"    'texinfo-insert-@samp)
+    (define-key map "\C-c\C-cq"    'texinfo-insert-@quotation)
+    (define-key map "\C-c\C-co"    'texinfo-insert-@noindent)
+    (define-key map "\C-c\C-cn"    'texinfo-insert-@node)
+    (define-key map "\C-c\C-cm"    'texinfo-insert-@email)
+    (define-key map "\C-c\C-ck"    'texinfo-insert-@kbd)
+    (define-key map "\C-c\C-ci"    'texinfo-insert-@item)
+    (define-key map "\C-c\C-cf"    'texinfo-insert-@file)
+    (define-key map "\C-c\C-cx"    'texinfo-insert-@example)
+    (define-key map "\C-c\C-ce"    'texinfo-insert-@end)
+    (define-key map "\C-c\C-cd"    'texinfo-insert-@dfn)
+    (define-key map "\C-c\C-cc"    'texinfo-insert-@code)
+    map))
+
+(easy-menu-define texinfo-mode-menu
+  texinfo-mode-map
+  "Menu used for `texinfo-mode'."
+  '("Texinfo"
+    ["Insert block"	texinfo-insert-block	t]
+    ;; ["Insert node"	texinfo-insert-@node	t]
+    "----"
+    ["Update All"	(lambda () (interactive) (texinfo-master-menu t))
+     :keys "\\[universal-argument] \\[texinfo-master-menu]"]
+    ["Update every node" texinfo-every-node-update t]
+    ["Update node"	texinfo-update-node	t]
+    ["Make Master menu"	texinfo-master-menu	t]
+    ["Make menu"	texinfo-make-menu	t]
+    ["Update all menus"	texinfo-all-menus-update t]
+    "----"
+    ["Show structure"	texinfo-show-structure	t]
+    ["Format region"	texinfo-format-region	t]
+    ["Format buffer"	texinfo-format-buffer	t]
+    ["Makeinfo region"	makeinfo-region		t]
+    ["Makeinfo buffer"	makeinfo-buffer		t]))
 
 
-;;; Texinfo mode
+(defun texinfo-filter (section list)
+  (let (res)
+    (dolist (x list) (if (eq section (cadr x)) (push (car x) res)))
+    res))
 
 (defvar texinfo-chapter-level-regexp
-  "chapter\\|unnumbered \\|appendix \\|majorheading\\|chapheading"
-  "Regular expression matching Texinfo chapter-level headings.
-This does not match `@node' and does not match the `@top' command.")
+  (regexp-opt (texinfo-filter 2 texinfo-section-list))
+  "Regular expression matching just the Texinfo chapter level headings.")
+
+;;; Texinfo mode
 
 ;;;###autoload
-(defun texinfo-mode ()
+(define-derived-mode texinfo-mode text-mode "Texinfo"
   "Major mode for editing Texinfo files.
 
   It has these extra commands:
@@ -480,22 +566,15 @@ Top node, is accompanied by some kind of section line, such as an
 If the file has a `top' node, it must be called `top' or `Top' and
 be the first node in the file.
 
-Entering Texinfo mode calls the value of text-mode-hook, and then the
-value of texinfo-mode-hook."
-  (interactive)
-  (text-mode)
-  (setq mode-name "Texinfo")
-  (setq major-mode 'texinfo-mode)
-  (use-local-map texinfo-mode-map)
-  (set-syntax-table texinfo-mode-syntax-table)
-  (make-local-variable 'page-delimiter)
-  (setq page-delimiter
-        (concat
-         "^@node [ \t]*[Tt]op\\|^@\\("
-         texinfo-chapter-level-regexp
-         "\\)"))
+Entering Texinfo mode calls the value of `text-mode-hook', and then the
+value of `texinfo-mode-hook'."
+  (set (make-local-variable 'page-delimiter)
+       (concat
+	"^@node [ \t]*[Tt]op\\|^@\\("
+	texinfo-chapter-level-regexp
+	"\\)\\>"))
   (make-local-variable 'require-final-newline)
-  (setq require-final-newline t)
+  (setq require-final-newline mode-require-final-newline)
   (make-local-variable 'indent-tabs-mode)
   (setq indent-tabs-mode nil)
   (make-local-variable 'paragraph-separate)
@@ -503,28 +582,39 @@ value of texinfo-mode-hook."
 	(concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-separate))
   (make-local-variable 'paragraph-start)
   (setq paragraph-start (concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-start))
+  (make-local-variable 'sentence-end-base)
+  (setq sentence-end-base
+	"\\(@\\(end\\)?dots{}\\|[.?!]\\)[]\"'”)}]*")
   (make-local-variable 'adaptive-fill-mode)
   (setq adaptive-fill-mode nil)
   (make-local-variable 'fill-column)
-  (setq fill-column 72)
+  (setq fill-column 70)
   (make-local-variable 'comment-start)
   (setq comment-start "@c ")
   (make-local-variable 'comment-start-skip)
-  (setq comment-start-skip "@c +")
+  (setq comment-start-skip "@c +\\|@comment +")
   (make-local-variable 'words-include-escapes)
   (setq words-include-escapes t)
   (make-local-variable 'imenu-generic-expression)
   (setq imenu-generic-expression texinfo-imenu-generic-expression)
   (setq imenu-case-fold-search nil)
   (make-local-variable 'font-lock-defaults)
-  (setq font-lock-defaults '(texinfo-font-lock-keywords t))
-  (make-local-variable 'outline-regexp)
-  (setq outline-regexp
-        (concat "@\\("
-                (mapconcat 'car texinfo-section-list "\\>\\|")
-                "\\>\\)"))
-  (make-local-variable 'outline-level)
-  (setq outline-level 'texinfo-outline-level)
+  (setq font-lock-defaults
+	'(texinfo-font-lock-keywords nil nil nil backward-paragraph
+				     (font-lock-syntactic-keywords
+				      . texinfo-font-lock-syntactic-keywords)))
+  (set (make-local-variable 'parse-sexp-lookup-properties) t)
+
+  ;; Outline settings.
+  (set (make-local-variable 'outline-heading-alist)
+       ;; We should merge outline-heading-alist and texinfo-section-list
+       ;; but in the mean time, let's just generate one from the other.
+       (mapcar (lambda (x) (cons (concat "@" (car x)) (cadr x)))
+	       texinfo-section-list))
+  (set (make-local-variable 'outline-regexp)
+       (concat (regexp-opt (mapcar 'car outline-heading-alist) t)
+	       "\\>"))
+
   (make-local-variable 'tex-start-of-header)
   (setq tex-start-of-header "%\\*\\*start")
   (make-local-variable 'tex-end-of-header)
@@ -533,63 +623,107 @@ value of texinfo-mode-hook."
   (setq tex-first-line-header-regexp "^\\\\input")
   (make-local-variable 'tex-trailer)
   (setq tex-trailer "@bye\n")
-  (run-hooks 'text-mode-hook 'texinfo-mode-hook))
+
+  ;; Prevent filling certain lines, in addition to ones specified
+  ;; by the user.
+  (let ((prevent-filling "^@\\(def\\|multitable\\)"))
+    (set (make-local-variable 'auto-fill-inhibit-regexp)
+	 (if (null auto-fill-inhibit-regexp)
+	     prevent-filling
+	   (concat auto-fill-inhibit-regexp "\\|" prevent-filling)))))
+
 
 
 ;;; Insert string commands
 
-;; Keep as concatinated lists for ease of maintenance
-(defconst texinfo-environment-regexp
-  (concat
-   "^@"
-   "\\("
-   "cartouche\\|"
-   "display\\|"
-   "end\\|"
-   "enumerate\\|"
-   "example\\|"
-   "f?table\\|"
-   "flushleft\\|"
-   "flushright\\|"
-   "format\\|"
-   "group\\|"
-   "ifhtml\\|"
-   "ifinfo\\|"
-   "iftex\\|"
-   "ignore\\|"
-   "itemize\\|"
-   "lisp\\|"
-   "macro\\|"
-   "multitable\\|"
-   "quotation\\|"
-   "smalldisplay\\|"
-   "smallexample\\|"
-   "smallformat\\|"
-   "smalllisp\\|"
-   "tex"
-   "\\)")
-  "Regexp for environment-like TexInfo list commands.
-   Subexpression 1 is what goes into the corresponding `@end' statement.")
+(defvar texinfo-block-default "example")
+
+(define-skeleton texinfo-insert-block
+  "Create a matching pair @<cmd> .. @end <cmd> at point.
+Puts point on a blank line between them."
+  (setq texinfo-block-default
+	(completing-read (format "Block name [%s]: " texinfo-block-default)
+			 texinfo-environments
+			 nil nil nil nil texinfo-block-default))
+  \n "@" str \n _ \n "@end " str \n)
+
+(defun texinfo-inside-macro-p (macro &optional bound)
+  "Non-nil if inside a macro matching the regexp MACRO."
+  (condition-case nil
+      (save-excursion
+	(save-restriction
+	  (narrow-to-region bound (point))
+	  (while (progn
+		   (up-list -1)
+		   (not (condition-case nil
+			    (save-excursion
+			      (backward-sexp 1)
+			      (looking-at macro))
+			  (scan-error nil)))))
+	  t))
+    (scan-error nil)))
+
+(defun texinfo-inside-env-p (env &optional bound)
+  "Non-nil if inside an environment matching the regexp @ENV."
+  (save-excursion
+    (and (re-search-backward (concat "@\\(end\\s +\\)?" env) bound t)
+	 (not (match-end 1)))))
+
+(defvar texinfo-enable-quote-macros "@\\(code\\|samp\\|kbd\\)\\>")
+(defvar texinfo-enable-quote-envs '("example\\>" "lisp\\>"))
+(defun texinfo-insert-quote (&optional arg)
+  "Insert the appropriate quote mark for Texinfo.
+Usually inserts the value of `texinfo-open-quote' (normally ``) or
+`texinfo-close-quote' (normally ''), depending on the context.
+With prefix argument or inside @code or @example, inserts a plain \"."
+  (interactive "*P")
+  (let ((top (or (save-excursion (re-search-backward "@node\\>" nil t))
+		 (point-min))))
+    (if (or arg
+	    (= (preceding-char) ?\\)
+	    (save-excursion
+              ;; Might be near the start of a (narrowed) buffer.
+              (ignore-errors (backward-char (length texinfo-open-quote)))
+	      (when (or (looking-at texinfo-open-quote)
+			(looking-at texinfo-close-quote))
+		(delete-char (length texinfo-open-quote))
+		t))
+	    (texinfo-inside-macro-p texinfo-enable-quote-macros top)
+	    (let ((in-env nil))
+	      (dolist (env texinfo-enable-quote-envs in-env)
+		(if (texinfo-inside-env-p env top)
+		    (setq in-env t)))))
+	(self-insert-command (prefix-numeric-value arg))
+      (insert
+       (if (or (bobp)
+               (memq (char-syntax (preceding-char)) '(?\( ?> ?\s)))
+	   texinfo-open-quote
+	 texinfo-close-quote)))))
 
 ;; The following texinfo-insert-@end command not only inserts a SPC
 ;; after the @end, but tries to find out what belongs there.  It is
 ;; not very smart: it does not understand nested lists.
 
+(defun texinfo-last-unended-begin ()
+  (while (and (re-search-backward texinfo-environment-regexp)
+	      (looking-at "@end"))
+    (texinfo-last-unended-begin)))
+
+(defun texinfo-next-unmatched-end ()
+  (while (and (re-search-forward texinfo-environment-regexp)
+	      (save-excursion
+		(goto-char (match-beginning 0))
+		(not (looking-at "@end"))))
+    (texinfo-next-unmatched-end)))
+
 (defun texinfo-insert-@end ()
   "Insert the matching `@end' for the last Texinfo command that needs one."
   (interactive)
-  (let ((depth 1) string)
-    (save-excursion
-      (while (and (> depth 0)
-                  (re-search-backward texinfo-environment-regexp nil t))
-	(if (looking-at "@end")
-	    (setq depth (1+ depth))
-	  (setq depth (1- depth))))
-      (looking-at texinfo-environment-regexp)
-      (if (zerop depth)
-          (setq string
-                (buffer-substring (match-beginning 1)
-                                  (match-end 1)))))
+  (let ((string
+	 (ignore-errors
+	   (save-excursion
+	     (texinfo-last-unended-begin)
+	     (match-string 1)))))
     (insert "@end ")
     (if string (insert string "\n"))))
 
@@ -640,6 +774,20 @@ The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "dfn" arg))
 
+(defun texinfo-insert-@email (&optional arg)
+  "Insert a `@email{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
+  (interactive "P")
+  (texinfo-insert-@-with-arg "email" arg))
+
+(defun texinfo-insert-@emph (&optional arg)
+  "Insert a `@emph{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
+  (interactive "P")
+  (texinfo-insert-@-with-arg "emph" arg))
+
 (defun texinfo-insert-@example ()
   "Insert the string `@example' in a Texinfo buffer."
   (interactive)
@@ -653,10 +801,18 @@ The default is not to surround any existing words with the braces."
   (texinfo-insert-@-with-arg "file" arg))
 
 (defun texinfo-insert-@item ()
-  "Insert the string `@item' in a Texinfo buffer."
+  "Insert the string `@item' in a Texinfo buffer.
+If in a table defined by @table, follow said string with a space.
+Otherwise, follow with a newline."
   (interactive)
-  (insert "@item")
-  (newline))
+  (insert "@item"
+	  (if (equal (ignore-errors
+		      (save-excursion
+			(texinfo-last-unended-begin)
+			(match-string 1)))
+		     "table")
+	      ?\s
+	    ?\n)))
 
 (defun texinfo-insert-@kbd (&optional arg)
   "Insert a `@kbd{...}' command in a Texinfo buffer.
@@ -667,11 +823,12 @@ The default is not to surround any existing words with the braces."
 
 (defun texinfo-insert-@node ()
   "Insert the string `@node' in a Texinfo buffer.
-This also inserts on the following line a comment indicating
-the order of arguments to @node."
+Insert a comment on the following line indicating the order of
+arguments to @node.  Insert a carriage return after the comment line.
+Leave point after `@node'."
   (interactive)
-  (insert "@node \n@comment  node-name,  next,  previous,  up")
-  (forward-line -1)
+  (insert "@node \n@comment  node-name,  next,  previous,  up\n")
+  (forward-line -2)
   (forward-char 6))
 
 (defun texinfo-insert-@noindent ()
@@ -679,6 +836,11 @@ the order of arguments to @node."
   (interactive)
   (insert "@noindent\n"))
 
+(defun texinfo-insert-@quotation ()
+  "Insert the string `@quotation' in a Texinfo buffer."
+  (interactive)
+  (insert "@quotation\n"))
+
 (defun texinfo-insert-@samp (&optional arg)
   "Insert a `@samp{...}' command in a Texinfo buffer.
 A numeric argument says how many words the braces should surround.
@@ -686,9 +848,16 @@ The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "samp" arg))
 
-(defun texinfo-insert-@table (&optional arg)
-  "Insert the string `@table' in a Texinfo buffer."
+(defun texinfo-insert-@strong (&optional arg)
+  "Insert a `@strong{...}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
   (interactive "P")
+  (texinfo-insert-@-with-arg "strong" arg))
+
+(defun texinfo-insert-@table ()
+  "Insert the string `@table' in a Texinfo buffer."
+  (interactive)
   (insert "@table "))
 
 (defun texinfo-insert-@var (&optional arg)
@@ -697,15 +866,17 @@ A numeric argument says how many words the braces should surround.
 The default is not to surround any existing words with the braces."
   (interactive "P")
   (texinfo-insert-@-with-arg "var" arg))
+
+(defun texinfo-insert-@uref (&optional arg)
+  "Insert a `@uref{}' command in a Texinfo buffer.
+A numeric argument says how many words the braces should surround.
+The default is not to surround any existing words with the braces."
+  (interactive "P")
+  (texinfo-insert-@-with-arg "uref" arg))
+(defalias 'texinfo-insert-@url 'texinfo-insert-@uref)
 
 ;;; Texinfo file structure
 
-;; These are defined in texnfo-upd.el.  defvars here avoid warnings.
-(defvar texinfo-section-types-regexp)
-(defvar texinfo-section-level-regexp)
-(defvar texinfo-subsection-level-regexp)
-(defvar texinfo-subsubsection-level-regexp)
-
 (defun texinfo-show-structure (&optional nodes-too)
   "Show the structure of a Texinfo file.
 List the lines in the file that begin with the @-sign commands for
@@ -717,78 +888,71 @@ with @-sign commands for @chapter, @section, and the like, and list
 
 Lines with structuring commands beginning in them are displayed in
 another buffer named `*Occur*'.  In that buffer, you can move point to
-one of those lines and then use \\<occur-mode-map>\\[occur-mode-goto-occurrence],
+one of those lines and then use
+\\<occur-mode-map>\\[occur-mode-goto-occurrence],
 to jump to the corresponding spot in the Texinfo source file."
 
   (interactive "P")
-  (require 'texnfo-upd)
-  (save-excursion
+  ;; First, remember current location
+  (let (current-location)
+    (save-excursion
+      (end-of-line)            ; so as to find section on current line
+      (if (re-search-backward
+           ;; do not require `texinfo-section-types-regexp' in texnfo-upd.el
+           "^@\\(chapter \\|sect\\|subs\\|subh\\|unnum\\|major\\|chapheading \\|heading \\|appendix\\)"
+           nil t)
+          (setq current-location
+                (progn
+                  (beginning-of-line)
+                  (buffer-substring (point) (progn (end-of-line) (point)))))
+        ;; else point is located before any section command.
+        (setq current-location "tex")))
+    ;; Second, create and format an *Occur* buffer
+    (save-excursion
+      (goto-char (point-min))
+      (occur (concat "^\\(?:" (if nodes-too "@node\\>\\|")
+		     outline-regexp "\\)")))
+    (pop-to-buffer "*Occur*")
     (goto-char (point-min))
-    (if nodes-too
-        (occur (concat "\\(^@node\\)\\|" texinfo-section-types-regexp))
-      (occur texinfo-section-types-regexp)))
-  (pop-to-buffer "*Occur*")
-  (goto-char (point-min))
-  (flush-lines "-----")
-  ;; Now format the "*Occur*" buffer to show the structure.
-  ;; Thanks to ceder@signum.se (Per Cederqvist)
-  (goto-char (point-max))
-  (let ((margin 5))
-    (while (re-search-backward "^ *[0-9]*:" nil 0)
-      (re-search-forward ":")
-      (setq margin
-            (cond
-             ((looking-at
-               (concat "@\\(" texinfo-chapter-level-regexp "\\)")) 5)
-             ;; ((looking-at "@chapter ") 5)
-             ;; ((looking-at "@unnumbered ") 5)
-             ;; ((looking-at "@appendix ") 5)
-             ;; ((looking-at "@majorheading ") 5)
-             ;; ((looking-at "@chapheading ") 5)
+    (let ((inhibit-read-only t))
+      (flush-lines "-----")
+      ;; Now format the "*Occur*" buffer to show the structure.
+      ;; Thanks to ceder@signum.se (Per Cederqvist)
+      (goto-char (point-max))
+      (let (level)
+        (while (re-search-backward "^ *[0-9]*:@\\(\\sw+\\)" nil 0)
+          (goto-char (1- (match-beginning 1)))
+          (setq level
+                (or (cadr (assoc (match-string 1) texinfo-section-list)) 2))
+          (indent-to-column (+ (current-column) (* 4 (- level 2))))
+          (beginning-of-line))))
+    ;; Third, go to line corresponding to location in source file
+    ;; potential bug: two exactly similar `current-location' lines ...
+    (goto-char (point-min))
+    (re-search-forward current-location nil t)
+    (beginning-of-line)
+    ))
 
-             ((looking-at
-               (concat "@\\(" texinfo-section-level-regexp "\\)")) 9)
-             ;; ((looking-at "@section ") 9)
-             ;; ((looking-at "@unnumberedsec ") 9)
-             ;; ((looking-at "@appendixsec ") 9)
-             ;; ((looking-at "@heading ") 9)
-
-             ((looking-at
-               (concat "@\\(" texinfo-subsection-level-regexp "\\)")) 13)
-             ;; ((looking-at "@subsection ") 13)
-             ;; ((looking-at "@unnumberedsubsec ") 13)
-             ;; ((looking-at "@appendixsubsec ") 13)
-             ;; ((looking-at "@subheading ") 13)
-
-             ((looking-at
-               (concat "@\\(" texinfo-subsubsection-level-regexp "\\)")) 17)
-             ;; ((looking-at "@subsubsection ") 17)
-             ;; ((looking-at "@unnumberedsubsubsec ") 17)
-             ;; ((looking-at "@appendixsubsubsec ") 17)
-             ;; ((looking-at "@subsubheading ") 17)
-             (t margin)))
-      (indent-to-column margin)
-      (beginning-of-line))))
 
 ;;; The  tex  and  print  function definitions:
 
 (defcustom texinfo-texi2dvi-command "texi2dvi"
-  "*Command used by `texinfo-tex-buffer' to run TeX and texindex on a buffer."
+  "Command used by `texinfo-tex-buffer' to run TeX and texindex on a buffer."
   :type 'string
   :group 'texinfo)
 
 (defcustom texinfo-tex-command "tex"
-  "*Command used by `texinfo-tex-region' to run TeX on a region."
+  "Command used by `texinfo-tex-region' to run TeX on a region."
   :type 'string
   :group 'texinfo)
 
 (defcustom texinfo-texindex-command "texindex"
-  "*Command used by `texinfo-texindex' to sort unsorted index files."
+  "Command used by `texinfo-texindex' to sort unsorted index files."
   :type 'string
   :group 'texinfo)
 
 (defcustom texinfo-delete-from-print-queue-command "lprm"
-  "*Command string used to delete a job from the line printer queue.
+  "Command string used to delete a job from the line printer queue.
 Command is used by \\[texinfo-delete-from-print-queue] based on
 number provided by a previous \\[tex-show-print-queue]
 command."
@@ -813,14 +977,17 @@ The value of `texinfo-tex-trailer' is appended to the temporary file after the r
   (interactive "r")
   (require 'tex-mode)
   (let ((tex-command texinfo-tex-command)
-	(tex-trailer "@bye\n"))
+	(tex-trailer texinfo-tex-trailer))
     (tex-region beg end)))
 
 (defun texinfo-tex-buffer ()
   "Run TeX on visited file, once or twice, to make a correct `.dvi' file."
   (interactive)
   (require 'tex-mode)
-  (let ((tex-command texinfo-texi2dvi-command))
+  (let ((tex-command texinfo-texi2dvi-command)
+	;; Disable tex-start-options-string.  texi2dvi would not
+	;; understand anything specified here.
+	(tex-start-options-string ""))
     (tex-buffer)))
 
 (defun texinfo-texindex ()
@@ -883,4 +1050,5 @@ You are prompted for the job number (use a number shown by a previous
 
 (provide 'texinfo)
 
+;; arch-tag: 005d7c38-43b9-4b7d-aa1d-aea69bae73e1
 ;;; texinfo.el ends here
