@@ -1,7 +1,24 @@
+/* Copyright    Massachusetts Institute of Technology    1985	*/
+
 #include "copyright.h"
 
-/* $Header: /cvsroot/emacs/emacs/oldXMenu/FindSel.c,v 1.1 1999/10/03 19:35:03 fx Exp $ */
-/* Copyright    Massachusetts Institute of Technology    1985	*/
+/*
+Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+  2009, 2010  Free Software Foundation, Inc.
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 /*
  * XMenu:	MIT Project Athena, X Window system menu package
@@ -70,3 +87,6 @@ XMenuFindSelection(menu, p_num, label)
     _XMErrorCode = XME_S_NOT_FOUND;
     return (XM_FAILURE);
 }
+
+/* arch-tag: 564a4a95-9ab0-4580-b05f-6970c4b25dd4
+   (do not change this comment) */
